package ac.essex.ooechs.imaging.commons.apps.facecropping;

import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.Pixel;
import ac.essex.ooechs.imaging.commons.apps.training.strategies.NodeSaver;
import ac.essex.ooechs.imaging.commons.apps.training.TrainingDataGUI;
import ac.essex.ooechs.imaging.commons.util.ImageFilenameFilter;
import ac.essex.ooechs.imaging.commons.util.panels.ImagePanel;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.awt.event.*;
import java.awt.*;
import java.util.Vector;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 28-Jul-2008
 * @version 1.0
 */
public class AccurateFaceCroppingGUI extends JFrame {

    protected Vector<File> files;
    protected int cursor = 0;
    protected PixelLoader currentImage;
    protected String currentFilename;
    protected File currentDirectory;
    protected CroppingPanel panel;

    public static void main(String[] args) {
        new AccurateFaceCroppingGUI(new File("/home/ooechs/Desktop/faces"));
    }

    public AccurateFaceCroppingGUI(File directory) {

        getFiles(directory);

        addKeyListener(new KeyAdapter() {

            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_SPACE) {
                    next();
                }
                if (e.getKeyCode() == KeyEvent.VK_UP) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.y--;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_DOWN) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.y++;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_LEFT) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.x--;
                        panel.repaint();
                    }
                }
                if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
                    if (panel.currentPixel != null) {
                        panel.currentPixel.x++;
                        panel.repaint();
                    }
                }
                
            }

        });

        panel = new CroppingPanel();
        getContentPane().add(panel);

        setTitle("Accurate Face Cropping");
        setSize(640, 480);
        setVisible(true);
        setLocationRelativeTo(null);

        next();


    }

    public void getFiles(File directory) {
        this.currentDirectory = directory;
        files = new Vector<File>(50);
        File[] fileList = directory.listFiles(new ImageFilenameFilter());
        for (int i = 0; i < fileList.length; i++) {
            File file = fileList[i];
            files.add(file);
        }
    }

    public void next() {
        try {
            currentImage = new PixelLoader(files.elementAt(cursor));
            currentFilename = currentImage.getFilename();
            panel.setImage(currentImage);
            cursor++;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    class CroppingPanel extends ImagePanel {

        public Pixel currentPixel;

        int mode = 0;

        Pixel leftEye;
        Pixel rightEye;
        Pixel centerPoint;
        Pixel mouth;

        double eyeAngle;

        int x, y;

        double eyeDx, eyeDy;

        public CroppingPanel() {

            addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {

                    if (mode == 0) {
                        leftEye = new Pixel(getX(e), getY(e));
                        currentPixel = leftEye;
                        mode++;
                        repaint();
                        return;
                    }

                    if (mode == 1) {
                        rightEye = new Pixel(getX(e), getY(e));
                        mode++;
                        calculateCenterPoint();
                        repaint();
                        currentPixel = rightEye;
                        return;
                    }

                    if (mode == 2) {
                        System.out.println("Saving nodes");
                        // save the nodes
                        Vector<Pixel> nodes = new Vector<Pixel>();
                        nodes.add(leftEye);
                        nodes.add(rightEye);
                        nodes.add(mouth);
                        try {
                            NodeSaver.save(nodes, new File(currentDirectory, currentFilename + ".nodes"));
                        } catch (IOException eee) {
                            eee.printStackTrace();
                        }
                        next();
                    }


                }
            });

            addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseMoved(MouseEvent e) {

                    x = getX(e);
                    y = getY(e);

                    if (mode > 1) {
                        calculateCenterPoint();
                        repaint();
                    }

                }
            });

        }

        public void calculateCenterPoint() {
            eyeDx = rightEye.x - leftEye.x;
            eyeDy = rightEye.y - leftEye.y;

            // find the center point between the eyes
            double c1x = leftEye.x + (eyeDx / 2);
            double c1y = leftEye.y + (eyeDy / 2);

            centerPoint = new Pixel((int) c1x, (int) c1y);

            // angle between the mouthpoint and center point
            eyeAngle = Math.tan(eyeDy / eyeDx);
        }

        public void setImage(PixelLoader image) {
            super.setImage(image);
            mode = 0;
            leftEye = null;
            rightEye = null;
            mouth = null;
            centerPoint = null;
        }

        public void paintComponent(Graphics g) {

            super.paintComponent(g);

            if (leftEye != null) paintNode(leftEye, g);
            if (rightEye != null) paintNode(rightEye, g);
            if (centerPoint != null) paintNode(centerPoint, g);

            if (mode > 1) {
                // draw a line between the left and right eye
                drawLine(g, leftEye.x, leftEye.y, rightEye.x, rightEye.y);

                // and a line down to the mouth
                int adjacent = y - centerPoint.y;
                int dx = (int) (adjacent * Math.tan(-eyeAngle));

                mouth = new Pixel(centerPoint.x + dx, centerPoint.y + adjacent);

                drawLine(g, centerPoint.x, centerPoint.y, mouth.x, mouth.y);

                // draw a perpendicular line
                drawLine(g, (int) (mouth.x - (eyeDx/2)), (int)  (mouth.y - (eyeDy/2)), (int) (mouth.x + (eyeDx/2)), (int) (mouth.y + (eyeDy/2)));

            }

            if (mouth != null) paintNode(mouth, g);

        }

        public void paintNode(Pixel node, Graphics g) {
            int crosshairSize = 5;
            g.setColor(Color.WHITE);
            drawLine(g, node.x, node.y - crosshairSize, node.x, node.y + crosshairSize);
            drawLine(g, node.x + crosshairSize, node.y, node.x - crosshairSize, node.y);
        }

    }


}
