package ac.essex.ooechs.imaging.commons.apps.training.strategies;

import ac.essex.ooechs.imaging.commons.util.Region;
import ac.essex.ooechs.imaging.commons.Pixel;
import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.ImageWindow;
import ac.essex.ooechs.imaging.commons.apps.training.TrainingDataGUI;
import ac.essex.ooechs.imaging.commons.subpixel.SubPixel;

import java.util.Vector;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 * This strategy is from Rowley, Bajula et al for their neural network
 * face detection problem.
 *
 * @author Olly Oechsle, University of Essex, Date: 23-Oct-2006
 * @version 1.0
 */
public class RowleyFaceCroppingStrategy implements CroppingStrategy {

    static boolean dontSaveImages = true;

    public static void main(String[] args) throws Exception {
        File directory = new File("/home/ooechs/Desktop/faces");
        File[] files = directory.listFiles();
        dontSaveImages = false;
        for (int i = 0; i < files.length; i++) {
            File file = files[i];
            File nodeFile = new File(file.getParentFile(), file.getName() + ".nodes");
            if (nodeFile.exists()) {
                PixelLoader image = new PixelLoader(file);
                Vector<Pixel> nodes = NodeSaver.load(nodeFile);
                //if (file.getName().indexOf("close") > -1) {
                System.out.println("Processing image");
                new RowleyFaceCroppingStrategy().getCroppingRegion(image, nodes.elementAt(0), nodes.elementAt(1), nodes.elementAt(2));
                //}
            }
        }
    }

    public String getName() {
        return "Rowley Face";
    }

    public String getDescription() {
        return "Click on the center of the left and right eyes, then on the middle of the upper lip";
    }

    public int countNodes() {
        return 3;
    }

    public double getAspectRatio() {
        return 1.0;
    }

    public Region getCropRegion(BufferedImage image, Vector<Pixel> nodes) throws StrategyException {

        if (nodes.size() != countNodes()) {
            throw new StrategyException("Cannot crop, I need an eye position first!");
        }

        Pixel eye1 = nodes.elementAt(0);
        Pixel eye2 = nodes.elementAt(1);
        Pixel mouth = nodes.elementAt(2);

        if (TrainingDataGUI.currentDirectory != null) {
            try {
                NodeSaver.save(nodes, new File(TrainingDataGUI.currentDirectory, TrainingDataGUI.currentFilename + ".nodes"));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return getCroppingRegion(new PixelLoader(image), eye1, eye2, mouth);

    }

    public Region getCroppingRegion(PixelLoader image, Pixel eye1, Pixel eye2, Pixel mouth) {

            Pixel leftEye = eye1;
            Pixel rightEye = eye2;

        // and find center point
        double eyeDx = rightEye.x - leftEye.x;
        double eyeDy = rightEye.y - leftEye.y;

        // find the center point between the eyes
        double c1x = leftEye.x + (eyeDx / 2);
        double c1y = leftEye.y + (eyeDy / 2);

        // find the angle between the mouth point and the center point
        double c1m1Dy = c1y - mouth.y;
        double c1m1Dx = c1x - mouth.x;

        // angle between the mouthpoint and center point
        double c1m1Angle = Math.tan(eyeDy / eyeDx);

        // distance between mouth point and center point
        double c1m1Dist = Math.sqrt((c1m1Dx * c1m1Dx) + (c1m1Dy * c1m1Dy));

        // crop the image
        double halfCropWidth = c1m1Dist * 2;
        int cropWidth = (int) (halfCropWidth * 2);

        double rotateAmount = c1m1Angle;


        if (!dontSaveImages) {

        // Now crop the image perfectly
        // the image needs to be 20x20 pixels wide
        // the distance between the mouth and eyes should be 12px (aka 60% of the height)

        // we already know the distance between the mouth and the eyes
        double imageSize = (int) (c1m1Dist / (12d / 20d));

        for (double angle = -5; angle <= 5; angle += 5) {

            double angleInRadians = Math.toRadians(angle);

            // Now rotate and crop the image
            PixelLoader rotated = SubPixel.extractRectangle(image, mouth.x, mouth.y, cropWidth, cropWidth, rotateAmount);

            double wLeft = halfCropWidth - (imageSize / 2);
            double wTop = halfCropWidth - c1m1Dist - (imageSize * (1 / 20)) - (imageSize / 4);
                        try {

            //PixelLoader cropped = rotated.getSubImage(new ImageWindow((int) wLeft, (int) wTop, (int) imageSize, (int) imageSize));
            PixelLoader cropped2 = SubPixel.extractRectangle(rotated, halfCropWidth, halfCropWidth - (c1m1Dist * 0.5), (int) imageSize, (int) imageSize, angleInRadians);


                            cropped2.normalise();


                //rotated.saveAs("/home/ooechs/Desktop/rotated.bmp");
                cropped2.saveAs("/home/ooechs/Desktop/faces/cropped/" + image.getFilename() + angle + ".bmp");
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
            
        }



        // build crop region around the centerY;

        // width is going to be twice the eye1 distance.
        // height is width / aspect ratio

        double width = eyeDx * 2.0d;
        double height = width / getAspectRatio();

        double left = c1x - (width / 2);
        double top = c1y - (height * 0.5);

        return new Region((int) left, (int) top, (int) width, (int) height);

    }




}
