package ac.essex.ooechs.imaging.commons.colourconstancy;

import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.ColourChannels;
import ac.essex.ooechs.imaging.commons.ColourConvertor;
import ac.essex.ooechs.imaging.commons.fast.FastStatistics;

import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * <p>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 09-Oct-2008
 * @version 1.0
 */
public class ColourConstancyTester {

    public static void main(String[] args) throws Exception {
        new ColourConstancyTester();
    }

    public ColourConstancyTester() throws Exception {
        PixelLoader image = new PixelLoader("/home/ooechs/Desktop/exposure/underexposed.jpg");
        new IlluminationTransform().transform(image).saveAs("/home/ooechs/Desktop/exposure/improved.jpg");
        //new WhitePatchRetinexTransform().transform(image).saveAs("/home/ooechs/Desktop/whiteatchretinex.jpg");

        //PixelLoader blueCube = new PixelLoader("/home/ooechs/Desktop/blue-cube.jpg");
        //PixelLoader yellowCube = new PixelLoader("/home/ooechs/Desktop/yellow-cube.jpg");
        //new GreyWorldAssumptionTransform().transform(blueCube).saveAs("/home/ooechs/Desktop/gwr-blue-cube.jpg");
        //new GreyWorldAssumptionTransform().transform(yellowCube).saveAs("/home/ooechs/Desktop/gwr-yelow-cube.jpg");

        //new WhitePatchRetinexTransform().transform(blueCube).saveAs("/home/ooechs/Desktop/wp-blue-cube.jpg");
        //new WhitePatchRetinexTransform().transform(yellowCube).saveAs("/home/ooechs/Desktop/wp-yellow-cube.jpg");

    }

    class IlluminationTransform extends ImageTransformer {

        /**
         * Returns the RGB value of the new colour.
         */
        public int transform(PixelLoader image, int x, int y) {

            float br = image.get3x3Stats(x, y, ColourChannels.RED, 10).getMean();
            float bg = image.get3x3Stats(x, y, ColourChannels.GREEN, 10).getMean();
            float bb = image.get3x3Stats(x, y, ColourChannels.BLUE, 10).getMean();

            float illumination = image.get3x3Stats(x, y, ColourChannels.LIGHTNESS).getMean();

            // the factor should aim to bring the illumination closer to grey
            float factor = 128 / illumination;

            // reduce the factor
            factor = (factor + 1f) / 2f;

            if (factor < 1) factor = 1;

            float maxR = Math.min(255, br * factor);
            float maxG = Math.min(255, bg * factor);
            float maxB = Math.min(255, bb * factor);

            int r = (int) ((image.getRed(x, y) / br) * maxR);
            int g = (int) ((image.getGreen(x, y) / bg) * maxG);
            int b = (int) ((image.getBlue(x, y) / bb) * maxB);

            if (r > 255) r = 255;
            if (r < 0) r = 0;
            if (g > 255) g = 255;
            if (g < 0) g = 0;
            if (b > 255) b = 255;
            if (b < 0) b = 0;
            return new Color(r, g, b).getRGB();
        }
    }

    class WhitePatchRetinexTransform extends ImageTransformer {

        int highestGrey = 0;
        float maxR, maxG, maxB;

        public void init(PixelLoader image, int x, int y) {
            if (image.getGreyValue(x, y) > highestGrey) {
                maxR = image.getRed(x, y);
                maxG = image.getGreen(x, y);
                maxB = image.getBlue(x, y);
                highestGrey = image.getGreyValue(x, y);
            }
/*            int r = image.getRed(x, y);
            if (r > maxR) maxR = r;
            int g = image.getGreen(x, y);
            if (g > maxG) maxG = g;
            int b = image.getBlue(x, y);
            if (b > maxB) maxB = b;*/
        }

        /**
         * Returns the RGB value of the new colour.
         */
        public int transform(PixelLoader image, int x, int y) {
            int r = (int) ((image.getRed(x, y) / maxR) * 255);
            if (r > 255) r = 255;
            if (r < 0) r = 0;
            int g = (int) ((image.getGreen(x, y) / maxG) * 255);
            if (g > 255) g = 255;
            if (g < 0) g = 0;
            int b = (int) ((image.getBlue(x, y) / maxB) * 255);
            if (b > 255) b = 255;
            if (b < 0) b = 0;
            return new Color(r, g, b).getRGB();
        }
    }

    class GreyWorldAssumptionTransform extends ImageTransformer {

        FastStatistics avr, avg, avb;

        public GreyWorldAssumptionTransform() {
            avr = new FastStatistics();
            avg = new FastStatistics();
            avb = new FastStatistics();
        }

        public void init(PixelLoader image, int x, int y) {
            avr.addData(image.getRed(x, y));
            avg.addData(image.getGreen(x, y));
            avb.addData(image.getBlue(x, y));
        }

        /**
         * Returns the RGB value of the new colour.
         */
        public int transform(PixelLoader image, int x, int y) {
            int r = (int) (image.getRed(x, y) - (avr.getMean()) + 128);
            if (r > 255) r = 255;
            if (r < 0) r = 0;
            int g = (int) (image.getGreen(x, y) - (avg.getMean()) + 128);
            if (g > 255) g = 255;
            if (g < 0) g = 0;
            int b = (int) (image.getBlue(x, y) - (avb.getMean()) + 128);
            if (b > 255) b = 255;
            if (b < 0) b = 0;
            return new Color(r, g, b).getRGB();
        }
    }

}
