package ac.essex.ooechs.imaging.commons.edge.hough.local;

import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.ooechs.imaging.commons.Pixel;

import java.awt.*;

/**
 * Performs a local hough transform for finding simple lines,
 * that is lines that pass through the origin,
 *
 * @author Olly Oechsle, University of Essex, Date: 24-Jun-2008
 * @version 1.0
 */
public class LocalHoughTransform {

    protected double maxTheta = Math.PI / 2;
    protected double minTheta = -Math.PI / 2;
    protected double thetaRange = maxTheta - minTheta;

    protected int[] houghArray;

    protected int points;

    protected int steps;




    public static void main(String[] args) throws Exception {
        //for (int i = 1; i <=4; i++) {

         int length = 10;
        int x = 11;
        int y = 11;

        LocalHoughTransform h = new LocalHoughTransform(36);
        PixelLoader image = new PixelLoader("/home/ooechs/Desktop/naughtyline.png");
        for (int iy = 0; iy < image.getHeight(); iy++) {
            for (int ix = 0; ix < image.getWidth(); ix++) {
                if (image.getGreyValue(ix, iy) > 0) {
                    h.addPoint(ix - x, iy - y);
                }
            }
        }


        double angle = h.getAngle();

                    int dy = (int) (length * Math.sin(angle));
                    int dx = (int) (length * Math.cos(angle));

                    // reject "weak" lines
                    if (h.getStrength() < 0.10/* || h.getPoints() < 5*/) {
                        //return;
                    }

                    Pixel lineStart = new Pixel(x + dx, y + dy);
                    Pixel lineEnd = new Pixel(x - dx, y - dy);

                    Graphics g = image.getBufferedImage().getGraphics();
                    g.setColor(Color.RED);
        g.drawLine(lineStart.x, lineStart.y, lineEnd.x, lineEnd.y);
        image.setRGB(x,y, Color.YELLOW.getRGB());

        image.saveAs("/home/ooechs/Desktop/naughtyline2.png");

        System.out.println(h.getAngle());
        System.out.println("Points: " + h.getPoints());
        System.out.println("Strength: " + h.getStrength());
        //}
    }

    public LocalHoughTransform() {
        this(8);
    }

    public LocalHoughTransform(int steps) {
        this.steps = steps;
        houghArray = new int[steps+1];
    }

    public void addPoint(int dx, int dy) {

        if (dx == 0 && dy == 0) return;

        // comes out in the range -pi / 2 > pi / 2
        double theta = Math.atan(dy / (double) dx);

        theta += Math.PI / 2;

        int index = (int) ((theta/thetaRange)*steps);

        houghArray[index]++;

        points++;

    }

    protected double mean = 0;
    protected int highest = 0;


    public double getAngle() {

        double highestAv = 0;

        double theta = 0;

        for (int i = 0; i < houghArray.length; i++) {

            // create a moving average - this smoothes the output
            // and ensures a more consistent result.
            double av = houghArray[i];
/*            if (i == 0) {
                av += houghArray[houghArray.length-1];
            } else {
                av += houghArray[i - 1];
            }

            if (i == houghArray.length - 1) {
                av += houghArray[0];
            } else {
                av += houghArray[i + 1];
            }

            av /= 3;*/

            //System.out.println(i + ", " + houghArray[i] + ", " + av);

            if (av > highestAv) {
                highestAv = av;
                highest = i;
                theta = ((i / (double) steps) * thetaRange) + minTheta;
            }
        }


        return theta;

    }

    public boolean isHorizontal(double angle) {
        return angle >= (Math.PI / 4) && angle <= (Math.PI / 4);
    }

    public double getStrength() {
        return highest / (double) points;
    }

    public int getPoints() {
        return points;
    }

}
