package ac.essex.ooechs.imaging.commons.signal;

import ac.essex.ooechs.imaging.commons.fast.FastStatistics;
import ac.essex.ooechs.imaging.commons.signal.SignalFeature;
import ac.essex.ooechs.imaging.commons.signal.Signal;

import java.util.Vector;

/**
 * Searches for trends in data. Useful.
 *
 * @author Olly Oechsle, University of Essex, Date: 15-Apr-2008
 * @version 1.0
 */
public class SignalProcessor {

    double[] data;

    public static void main(String[] args) {
        //SignalProcessor t = new SignalProcessor(new double[]{63.20000076293945,64.0,63.79999923706055,64.80000305175781,68.5999984741211,72.0,71.4000015258789,69.0,68.0,65.80000305175781,63.400001525878906,63.20000076293945,63.20000076293945,63.400001525878906,63.20000076293945,64.0,66.4000015258789,69.0,72.19999694824219,77.0,81.0,82.80000305175781,78.19999694824219,83.4000015258789,162.39999389648438,241.60000610351562,239.8000030517578,220.39999389648438,155.1999969482422,77.0,103.5999984741211,219.39999389648438,253.0,244.0,199.39999389648438,93.19999694824219,54.20000076293945,58.0,66.0,69.5999984741211,67.0,65.4000015258789,65.0,64.5999984741211,64.4000015258789,64.4000015258789,64.19999694824219,65.0,72.0});
       // t.getPeaks(0,49,200);
        //t.getTroughs(0, 49, 50);

        //System.out.println(t.getMaxWidth(0,49,200));
        //System.out.println(t.getPeakHeightRange(0, 49, 200));

    }

    public SignalProcessor(Signal signal) {
        this(signal.data);
    }

    public SignalProcessor(double[] data) {
        this.data = data;
    }

    public static int getMaxWidth(Vector<SignalFeature> features) {

        int maxWidth = 0;

        for (int i = 0; i < features.size(); i++) {
            SignalFeature peak = features.elementAt(i);
            if (peak.width > maxWidth)  {
                maxWidth = peak.width;
            }
        }

        return maxWidth;

    }

    public static int getMinWidth(Vector<SignalFeature> features) {

        int minWidth = Integer.MAX_VALUE;

        for (int i = 0; i < features.size(); i++) {
            SignalFeature peak = features.elementAt(i);
            if (peak.width < minWidth)  {
                minWidth = peak.width;
            }
        }

        return minWidth;

    }

    public static double getHeightRange(Vector<SignalFeature> features) {

        FastStatistics stats = new FastStatistics();

        for (int i = 0; i < features.size(); i++) {
            SignalFeature peak = features.elementAt(i);
            stats.addData((float) peak.height);
        }

        return stats.getRange();

    }

    public Vector<SignalFeature> getPeaks(int x1, int x2, double minHeight) {

        // find a peak - where the data crosses through the minheight
        Vector<SignalFeature> peaks = new Vector<SignalFeature>(2);

        double lastValue = 0;

        int peakStart = -1;
        double peakMaxValue = -1;
        int maxValuePoint = -1;

        for (int x = x1; x < x2; x++) {

            double currentValue = data[x];

            if (x > x1) {

                if (lastValue < minHeight && currentValue > minHeight) {
                    peakStart = x;
                    peakMaxValue = currentValue;
                    maxValuePoint = x;
                }

                if (peakStart != -1) {
                    if (peakMaxValue < currentValue) {
                        peakMaxValue = currentValue;
                        maxValuePoint = x;
                    }

                    if (lastValue > minHeight && currentValue < minHeight) {
                        int width = x - peakStart;
                        peaks.add(new SignalFeature(SignalFeature.PEAK, maxValuePoint, peakMaxValue, width));
                        peakStart = -1;
                    }

                }

            }

            lastValue = currentValue;

        }

        return peaks;

    }

    public Vector<SignalFeature> getTroughs(int x1, int x2, double maxHeight) {

        // find a peak - where the data crosses through the minheight
        Vector<SignalFeature> troughs = new Vector<SignalFeature>(2);

        double lastValue = 0;

        int troughStart = -1;
        double troughMinValue = -1;
        int minValuePoint = -1;

        for (int x = x1; x < x2; x++) {

            double currentValue = data[x];

            if (x > x1) {

                if (lastValue > maxHeight && currentValue < maxHeight) {
                    troughStart = x;
                    troughMinValue = currentValue;
                    minValuePoint = x;
                }

                if (troughStart != -1) {

                    if (troughMinValue < currentValue) {
                        troughMinValue = currentValue;
                        minValuePoint = x;
                    }

                    if (lastValue < maxHeight && currentValue > maxHeight) {
                        int width = x - troughStart;
                        troughs.add(new SignalFeature(SignalFeature.TROUGH, minValuePoint, troughMinValue, width));
                        troughStart = -1;
                    }

                }

            }

            lastValue = currentValue;

        }

        return troughs;

    }

}
