package ac.essex.ooechs.imaging.commons.thresholding;

/*
 * Mixture Modeling algorithm
 * Based on ImageJ plugin code by Christopher Mei (christopher.mei@sophia.inria.fr) and Maxime Dauphin
 *
 */
public class MixtureModellingThresholder extends Thresholder {

    final static int HMIN = 0;
    final static int HMAX = 256;

    public MixtureModellingThresholder(int max) {
        super(max);
    }

    public int getOptimalThreshold() {

        GrayLevelClassMixtureModelling model = new GrayLevelClassMixtureModelling(N, hist);

        int bestThreshold = 0;

        float error;
        float errorMin = Float.MAX_VALUE;
        float mu1 = 0, mu2 = 0;

        while (model.incrementThreshold()) {
            error = calculateError(model);

            if (error < errorMin) {
                errorMin = error;
                bestThreshold = model.getThreshold();
                mu1 = model.getMu1();
                mu2 = model.getMu2();
            }
        }

        model.setThreshold(bestThreshold);
        int t =  findThreshold((int) mu1, (int) mu2, model);

        System.out.println("Best threshold = " + bestThreshold);
        System.out.println("Find threshold = " + t);
        return t;

    }

    private int findThreshold(int mu1, int mu2, GrayLevelClassMixtureModelling classes) {
        float min = Float.MAX_VALUE;
        int threshold = 0;

        for (int i = mu1; i < mu2; i++) {
            float val = (float) Math.pow(classes.gammaDifference(i), 2);
            if (min > val) {
                min = val;
                threshold = i;
            }
        }

        return threshold;
    }

    private float calculateError(GrayLevelClassMixtureModelling classes) {
        float error = 0;
        for (int i = 0; i <= GrayLevelClassMixtureModelling.MAX; i++) {
            error += Math.pow(classes.gamma(i) - classes.histogram[i], 2);
        }
        return error / (GrayLevelClassMixtureModelling.MAX + 1);
    }

}


