package ac.essex.ooechs.imaging.commons.util.panels;

import ac.essex.ooechs.imaging.commons.PixelLoader;

import javax.swing.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import java.awt.event.MouseEvent;

/**
 * Panel onto which an image can be drawn easily.
 * Last updated September 4th 2008
 */
public class ImagePanel extends JPanel {

    public int zoom = 1;

    public BufferedImage image;

    protected boolean setDisplayCentered = false;

    protected boolean scale = false;

    public boolean display = true;

    public ImagePanel() {}

    public ImagePanel(PixelLoader image) {
        setImage(image);
    }

    public void setImage(BufferedImage image) {
        this.image = image;
        afterZoom();
        repaint();
    }

    public void setImage(PixelLoader image) {
        setImage(image.getBufferedImage());
    }

    public BufferedImage getImage() {
        return image;
    }

    public void setDisplayCentered(boolean centered) {
        this.setDisplayCentered = centered;
        if (!centered) {
            offsetX = 0;
            offsetY = 0;
        }
    }

    public void setScaling(boolean scale) {
        this.scale = scale;
    }

    protected int offsetX = 0;
    protected int offsetY = 0;

    public int getOffsetX() {
        return offsetX;
    }

    public int getOffsetY() {
        return offsetY;
    }

    public int getX(MouseEvent e) {
        return (e.getX() - offsetX) / zoom;
    }

    public int getY(MouseEvent e) {
        return (e.getY() - offsetY) / zoom;
    }



    public void drawImage(Graphics g, BufferedImage image) {
        if (!scale) {
            g.drawImage(image, offsetX, offsetY, image.getWidth() * zoom, image.getHeight() * zoom, this);
        } else {
            g.drawImage(image, 0, 0, getWidth(), getHeight(), this);
        }
    }

    public void drawRect(Graphics g, int x, int y, int width, int height) {
        g.drawRect((x + offsetX) * zoom, (y + offsetX) * zoom, width * zoom, height * zoom);
    }

    public void fillRect(Graphics g, int x, int y, int width, int height) {
        g.fillRect((x + offsetX) * zoom, (y + offsetX) * zoom, width * zoom, height * zoom);
    }

    public void drawLine(Graphics g, int x1, int y1, int x2, int y2) {
        g.drawLine((x1 + offsetX) * zoom, (y1 + offsetX) * zoom,(x2 + offsetX) * zoom, (y2 + offsetX) * zoom);
    }

    protected void afterZoom() {
        if (image == null) {
            setPreferredSize(new Dimension(getWidth(), getHeight()));
        } else {
            setPreferredSize(new Dimension(image.getWidth() * zoom, image.getHeight() * zoom));
        }
        revalidate();
    }

    public void zoomIn() {
        zoom += 2;
        if (zoom > 32) zoom = 32;
        afterZoom();
    }

    public void zoomOut() {
        zoom -= 2;
        if (zoom < 1) zoom = 1;
        afterZoom();
    }

    public void setImageNull() {
        image = null;
    }

    public void paintComponent(Graphics g) {

        if (image != null) {

            if (setDisplayCentered) {
                offsetX = (getWidth() - (image.getWidth() * zoom)) / 2;
                offsetY = (getHeight() - (image.getHeight() * zoom)) / 2;
            }

            if (display) {
                g.setColor(Color.BLACK);
                g.fillRect(0, 0, getWidth(), getHeight());
                drawImage(g, image);
            }

        } else {
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, getWidth(), getHeight());
        }
    }
}
