package ac.essex.ooechs.imaging.commons.window.TheMondayProject.trends;

/**
 * Represents a trend in which a value increases or decreases between two points.
 *
 * @author Olly Oechsle, University of Essex, Date: 14-Apr-2008
 * @version 1.0
 */
public class ChangeTrend extends Trend implements Comparable  {

    /**
     * The position at which the change takes place
     */
    public int x;

    public int s;

    public double minChange;

    /**
     * Encodes the point at which a feature changes
     */
    public ChangeTrend(int x, int s, double meanChange, double changeStdDeviation) {

        this.x = x;
        this.s = s;
        
        if (meanChange > 0) {
            this.minChange = meanChange - changeStdDeviation;
        } else {
            this.minChange = meanChange + changeStdDeviation;
        }

    }

    public boolean matches(double[] data) {
        try {

            double value1 = data[x];
            double value2 = data[x + s];

            double change = (value2 / value1) - 1;

            // if change is similar to what is expected
            if (minChange < 0) {
                if (change < minChange) return true;
            } else {
                if (change > minChange) return true;
            }

            return false;

        } catch (ArrayIndexOutOfBoundsException e) {
            return false;
        }

    }

    /**
     * Compares this object with the specified object for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.<p>
     */
    public int compareTo(Object o) {
        ChangeTrend other = (ChangeTrend) o;
        if (other.x > x) return -1;
        if (other.x < x) return +1;
        return 0;
    }    

}
