package ac.essex.ooechs.imaging.commons.window.TheMondayProject.tuesday;

import ac.essex.ooechs.imaging.commons.window.TheMondayProject.MondayProject;
import ac.essex.ooechs.imaging.commons.window.TheMondayProject.WindowData;
import ac.essex.ooechs.imaging.commons.fast.FastStatistics;
import ac.essex.ooechs.imaging.commons.signal.SignalProcessor;
import ac.essex.ooechs.imaging.commons.signal.SignalFeature;

import java.util.Vector;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 15-Apr-2008
 * @version 1.0
 */
public class TrendClassifier {

    public static void main(String[] args) throws Exception {
        MondayProject m = new MondayProject();
        m.load("/home/ooechs/Desktop/pipes/Foreground/", MondayProject.TRUE);
        m.load("/home/ooechs/Desktop/pipes/Background/", MondayProject.FALSE);
        m.load("/home/ooechs/Desktop/pipes/Unusual/", MondayProject.UNUSUAL);

        new TrendClassifier(m);
    }

    public TrendClassifier(MondayProject project) {

        Vector<WindowData> data;
        data = project.getData(MondayProject.TRUE);
        System.out.println("True data:");
        test(data);
        System.out.println("False data:");
        data = project.getData(MondayProject.FALSE);
        test(data);
        System.out.println("Unusual data:");
        data = project.getData(MondayProject.UNUSUAL);
        test(data);        

    }

    public void test(Vector<WindowData> data) {

        int[] histogram = new int[5];
        int total = 0;

        FastStatistics stats = new FastStatistics();

        for (int i = 0; i < data.size(); i++) {
            WindowData windowData = data.elementAt(i);
            SignalProcessor t = new SignalProcessor(windowData.data);

            int individualScore = 0;

            if (rule1(t)) individualScore++;
            if (rule2(t)) individualScore++;
            if (rule3(t)) individualScore++;
            if (rule4(t)) individualScore++;
            if (rule5(t)) individualScore++;

            stats.addData(individualScore);

            //System.out.println(windowData.classID + ", " + individualScore);

            histogram[individualScore-1]++;
            total ++;


        }

        System.out.println("Histogram:");
        for (int i = 0; i < histogram.length; i++) {
            int i1 = histogram[i];
            double percentage = i1 / (double) total;
            System.out.println((i + 1) + ", " + percentage);
        }

        //System.out.println(stats.getMean());
        //System.out.println(stats.getMin() + " - " +  stats.getMax());

    }


    public boolean rule1(SignalProcessor t) {
        return t.getPeaks(0, 49, 200).size() >= 2;
    }

    public boolean rule1a(SignalProcessor t) {
        return t.getPeaks(15, 35, 200).size() == 2;
    }

    public boolean rule2(SignalProcessor t) {
        Vector<SignalFeature> peaks = t.getPeaks(0,49,200);
        return t.getMaxWidth(peaks) < 5;
    }

    public boolean rule3(SignalProcessor t) {
        Vector<SignalFeature> peaks = t.getPeaks(0,49,200);
        return t.getHeightRange(peaks) < 16;
    }

    public boolean rule4(SignalProcessor t) {
        Vector<SignalFeature> troughs = t.getTroughs(0,49,100);
        return t.getMaxWidth(troughs) < 10;
    }

    public boolean rule5(SignalProcessor t) {
        return t.getPeaks(0,49,150).size() == t.getPeaks(0,49,200).size();
    }

}
