package ac.essex.ooechs.imaging.commons.window.util;

import ac.essex.ooechs.imaging.commons.window.data.Window;
import ac.essex.ooechs.imaging.commons.fast.FastStatistics;
import ac.essex.ooechs.imaging.commons.PixelLoader;

import java.util.Vector;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 20-Mar-2008
 * @version 1.0
 */
public class WindowFeatures {

    /**
     * @param image
     * @return
     * @throws Exception If the image can't be loaded.
     */
    public static double[] getFeatures_OLD(PixelLoader image, Window window) throws Exception {

        double[] values = new double[window.width];

        // move along the window one pixel at a time and get the summed intensity
        for (int wx = 0; wx < window.width; wx++) {
            int x = wx + window.x;
            FastStatistics total = new FastStatistics();
            for (int wy = 0; wy < window.height; wy++) {
                int y = wy + window.y;
                total.addData(image.getGreyValue(x, y));
            }
            values[wx] = total.getMean();
        }

        return values;

    }

    public static double[] getFeatures(PixelLoader image, Window window) throws Exception {

        Vector<Double> values = new Vector<Double>();

        int blockSize = 5;

        // move along the window one pixel at a time and get the summed intensity
        for (int wx = 0; wx < window.width-blockSize; wx+=blockSize) {

            FastStatistics total = new FastStatistics();
            for (int bx = 0; bx < 5; bx++) {
                int x = wx + window.x;
                for (int wy = 0; wy < window.height; wy++) {
                    int y = wy + window.y;
                    total.addData(image.getGreyValue(x, y));
                }
            }
            values.add((double) total.getMean());
            values.add((double) total.getStandardDeviation());
        }

        // do the same on the other axis
        for (int wy = 0; wy < window.height; wy++) {
            FastStatistics total = new FastStatistics();
            for (int bx = 0; bx < 5; bx++) {
                for (int wx = 0; wx < window.width; wx++) {
                    int y = wy + window.y;
                    int x = wx + window.x;
                    total.addData(image.getGreyValue(x, y));
                }
            }
            values.add((double) total.getMean());
            values.add((double) total.getStandardDeviation());
        }

        // convert vector into an array
        double[] v = new double[values.size()];
        for (int i = 0; i < values.size(); i++) {
            Double aDouble = values.elementAt(i);
            v[i] = aDouble;
        }

        return v;

    }

    public static double[] getFeaturesForMonday(PixelLoader image, Window window) throws Exception {

        Vector<Double> values = new Vector<Double>();

        int blockSize = 1;

        // move along the window one pixel at a time and get the summed intensity
        for (int wx = 0; wx < window.width-blockSize; wx+=blockSize) {

            FastStatistics total = new FastStatistics();
            for (int bx = 0; bx < 1; bx++) {
                int x = wx + window.x + bx;
                for (int wy = 0; wy < window.height; wy++) {
                    int y = wy + window.y;
                    total.addData(image.getGreyValue(x, y));
                }
            }
            values.add((double) total.getMean());
            //hist.add((double) total.getStandardDeviation());
        }

        // convert vector into an array
        double[] v = new double[values.size()];
        for (int i = 0; i < values.size(); i++) {
            Double aDouble = values.elementAt(i);
            v[i] = aDouble;
        }

        return v;

    }

}
